/**
	Symbolic colebrook equation and derivative
*/

#include <ginac/ginac.h>
#include <iostream>

#include "colebrook-sym.h"
#include "colebrook.h"
/*

nla-ginac - Tarjan-blocking NLA solver with autodiff
Copyright (C) 2005  John Pye

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

using namespace GiNaC;
using namespace std;


/// SATURATION STEAM TABLES: ENTHALPY

static ex f_ReEd_eval(const ex &Re, const ex &Ed){
	ex Re1 = Re.evalf();
	ex Ed1 = Ed.evalf();
	if(is_a<numeric>(Re1) && is_a<numeric>(Ed1)){
		double Re2 = ex_to<numeric>(Re1).to_double();
		double Ed2 = ex_to<numeric>(Ed1).to_double();
		return Colebrook::f_ReEd(Re2,Ed2);
	}else{
		return f_ReEd(Re,Ed).hold();
	}
}

static ex f_ReEd_diff(const ex &Re, const ex &Ed, unsigned diff_param){
	switch(diff_param){
		case 0:
			return f_ReEd_dRe(Re,Ed);
		case 1:
			return f_ReEd_dEd(Re,Ed);
		default:
			throw runtime_error("f_ReEd_diff: Invalid diff_param");
	}
}

REGISTER_FUNCTION(f_ReEd, eval_func(f_ReEd_eval).derivative_func(f_ReEd_diff));

/**
	The following derivatives were calculated using Maxima:
	1/sqrt(f)=-0.86*log(eD/3.7 + 2.51/Re/sqrt(f));
	diff(%o1)
	solve(%o2,DEL(f))
*/

static ex f_ReEd_dRe_eval(const ex &Re, const ex &Ed){
	ex f = f_ReEd(Re,Ed);
	return -(798682*pow(f,3/2)) / (50000*Ed*sqrt(f)*pow(Re,2)+(399341*sqrt(f)+464350)*Re);
}

static ex f_ReEd_dEd_eval(const ex &Re, const ex &Ed){
    ex f = f_ReEd(Re,Ed);
	return -(86000*pow(f,2)*pow(Re,2)) / (50000*Ed*sqrt(f)*pow(Re,2)+(399341*sqrt(f)+464350)*Re);

}

REGISTER_FUNCTION(f_ReEd_dRe, eval_func(f_ReEd_dRe_eval));
REGISTER_FUNCTION(f_ReEd_dEd, eval_func(f_ReEd_dEd_eval));

