#include <ginac/ginac.h>
#include <sstream>

#include "eqprocess.h"

using namespace std;
using namespace GiNaC;

/// ctor
EqProcess::EqProcess(SymbolicVariable *basevariable, const ex *baseequation)
		: basevariable(basevariable), baseequation(baseequation)
{
	//cerr << char(27) << "[2mEqProcess:: creating with variable " << char(27) << "[1m" << basevariable->getName() << char(27) << "[2m" << ", equation: " << char(27) << "[0m" << *baseequation << endl;
	
	// take a copy of the base equation
	currentequation = *baseequation;
	is_solved=false;
}

/// copy ctor
EqProcess::EqProcess(const EqProcess &e)
	: basevariable(e.basevariable), baseequation(e.baseequation){
	is_solved=e.is_solved;
	currentequation=e.currentequation;
}

/// assignment operator
EqProcess &
EqProcess::operator=(const EqProcess &e){
	this->basevariable = e.basevariable;
	this->baseequation = e.baseequation;
	this->currentequation = e.currentequation;
	this->is_solved = e.is_solved;
	this->solution = e.solution;
	return *this;
}

ex &
EqProcess::getCurrentEquation(){
	return currentequation;
}

const ex &
EqProcess::getBaseEquation() const{
	return *baseequation;
}
	
SymbolicVariable *
EqProcess::getBaseVariable(){
	return basevariable;
}


const double & 
EqProcess::getSolution() const{
	if(!is_solved){
		stringstream ss;
		ss << "EqProcess::getSolution: variable " << basevariable->getName() << " has not been solved yet.";
		throw runtime_error(ss.str());
	}
	return solution;
}

void 
EqProcess::setSolution(const double &solution){
	this->solution = solution;
	is_solved=true;
}

void 
EqProcess::subs(const GiNaC::relational &subst){
	try{
		//cerr << "Substituting ";
		//cerr << " into " << currentequation;
		currentequation = currentequation.subs(subst);
		//cerr << "... gives " << currentequation << endl;
	}catch(exception &e){
		stringstream ss;
		ss << "EqProcess::subs: " << e.what();
		throw runtime_error(ss.str());
	}
}
