#ifndef EQPROCESS_H
#define EQPROCESS_H

#include "symbolicvariable.h"

#include <ginac/ginac.h>

/// Equation solution in-process
/**
	A holder for variables / equations which are being
	solved in a TarjanSolver.

	There is a pointer to symbol and the original equation
	and a place to store the equation in its intermediate (partly solved) form.
	
	Finally, there is a place to store its solved value.

	The fact that variable and equation are stored in the same place
	reflects that fact that we have got our relationship matrix into 
	diagonal form, so there is a mapping of variables to equations already
	in place.
*/
class EqProcess{

public:
	EqProcess(SymbolicVariable *basevariable, const GiNaC::ex *baseequation);
	EqProcess(const EqProcess &e);
	EqProcess & operator=(const EqProcess &e);

	GiNaC::ex &getCurrentEquation();
	const GiNaC::ex &getBaseEquation() const;
	SymbolicVariable *getBaseVariable();

	const double& getSolution() const;
	void setSolution(const double &);

	void subs(const GiNaC::relational &subst);

private:
	SymbolicVariable *basevariable;
	const GiNaC::ex *baseequation;
	GiNaC::ex currentequation;
	bool is_solved;
	double solution;
};

#endif // EQPROCESS_H
