/*

nla-ginac - Tarjan-blocking NLA solver with autodiff
Copyright (C) 2005  John Pye

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#include "symbolicsystem.h"
#include "tarjansolver.h"
#include "nnessolver.h"
#include "roulette.h"
#include "units.h"
#include "iapws-sat.h"
#include "colebrook-sym.h"
#include "viscosity-sym.h"

using namespace std;
using namespace GiNaC;

class FrictionSystem: public SymbolicSystem{

	virtual void defineSystem(){

        CONST(eps,0.046e-3);
		CONST(L,600);
		CONST(D,0.06);
		CONST(A1,Pi * D*D/4);

		VARIABLE(p1,60. / 10);
		CONST(m1,0.01);
		CONST(T1,400+273.15);
		CONST(   rho1,801.7);
		//VARIABLE(h1,2400);

        VARIABLE(Re,8223);
        CONST(mu1,mu_Trho(T1,801.7));
		VARIABLE(f,0.02);

        VARIABLE(DP, -1/10);
		//VARIABLE(p2, 55./10);

		//VARIABLE(T2, 500);
		//VARIABLE(h2, 200);
		//VARIABLE(rho2, 801.7);

		LOWERB(f,0.008);
		UPPERB(f,0.1);
		//LOWERB(rho1,0);
		//UPPERB(rho1,1100);
		LOWERB(Re,700);
		UPPERB(Re,1e10);
		LOWERB(DP,-6);
		UPPERB(DP,0);
		//LOWERB(p2,0);
		//UPPERB(p2,100);
		//LOWERB(T2,273.15);
		//UPPERB(T2,1073.15);
		//LOWERB(h2,0);
		//UPPERB(h2,9000);
		//LOWERB(rho2,0);
		//UPPERB(rho2,1100);


		lst ff; ff =
			// Inlet properties
			p1 - water_p(T1,rho1) // p1
			//, h1 - water_h(T1,rho1) // p1
			, Re - (rho1 * m1 * 4) / (Pi * D * mu1) // Re
            , f - f_ReEd(Re,eps/D) // f
            , DP + 0.5 * f * L / D * pow(m1,2) / pow(A1,2) / rho1 // DP
			//, DP - (p2-p1)
			//, p2 - water_p(T2,rho2) // p1
			//, h2 - h1
			//, h2 - water_h(T2,rho2) // p1
   		;
        setExpressions(ff);

	}

	virtual void applySolverOptions(Solver *solver) const{
		// do bounds checking in NNES:
        //applyBounds(solver);

        //OPTION(ITSCLX,10);
        //OPTION(ITSCLF,9);
		//OPTION(NEWTON,0); OPTION(ABSNEW,0); OPTION(LINESR,0);
		OPTION(MAXIT,5000);
        OPTION(STOPCR,2);
        OPTION(JACTYP,0);
		OPTION(NJACCH,50);
        OPTION(FDTOLJ,8e-3);
        //OPTION(OUTPUT,0);
	}

};

int main(){
    FrictionSystem P;
	//Roulette *S = new Roulette();
    TarjanSolver *S = new TarjanSolver();
	//NNESSolver *S = NNESSolver::Instance();

    P.accept(S); // yes, i'll take a seat at the roulette table :)

}
