/*

nla-ginac - Tarjan-blocking NLA solver with autodiff
Copyright (C) 2005  John Pye

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
/**
	Test that the derivatives of the properties give correctly extrapolated values
*/
#include "iapws-sat.h"
#include <iostream>

using namespace std;
using namespace GiNaC;

int main(){

    symbol T("T"), rho("rho");

    ex rho1 = 0.02;
    ex rho0;

    cerr << "rho1 = " << rho1 << endl;

	ex T0 = 275;
	ex T1;
	ex dT = 5;
	ex p = water_p(T0,rho1);
	ex h = water_h(T0,rho1);
	ex s = water_s(T0,rho1);

//#define TEST_TEMP

#ifdef TEST_TEMP

	while(T0<1073){
		ex dpdT = water_p(T,rho1).diff(T).subs(T==T0);
		ex dhdT = water_h(T,rho1).diff(T).subs(T==T0);
		ex dsdT = water_s(T,rho1).diff(T).subs(T==T0);
		T1 = T0 + dT;
		ex p1 = p + dT * dpdT;
		ex h1 = h + dT * dhdT;
		ex s1 = s + dT * dsdT;

		cerr << "h = " << setw(8) << ex_to<numeric>(h).to_double() << "  ";
		cerr << "h1 = " << setw(8) << ex_to<numeric>(h1).to_double() << "  ";
		cerr << "T = " << setw(7) << ex_to<numeric>(T0).to_double() << "  ";
		cerr << "dh/dT = " << setw(8) << ex_to<numeric>(dhdT).to_double() << "  ";

		p = water_p(T1,rho1);
		h = water_h(T1,rho1);
		s = water_s(T1,rho1);

		cerr << "h = " << setw(8) << ex_to<numeric>(h).to_double() << "  ";

		numeric p_err = ex_to<numeric>( (p1 - p)/p * 100 );
		numeric h_err = ex_to<numeric>( (h1 - h)/h * 100 );
		numeric s_err = ex_to<numeric>( (s1 - s)/s * 100 );

		cerr << "err_p = " << setw(10) << setprecision(4) << p_err.to_double() << "%  ";
		cerr << "err_h = " << setw(10) << setprecision(4) << h_err.to_double() << "%  ";
		cerr << "err_s = " << setw(10) << setprecision(4) << s_err.to_double() << "%" << endl;
		T0 = T1;
	}
#endif

    rho0 = 880;
    T0 = 473.15;
    cerr << "DENSITY QUICK CHECK: " << endl;
    cerr << "T = " << T0 << ", rho = "<< rho0 << endl;
    cerr << "h = " << water_h(T0, rho0) << endl;
    cerr << "dhdrho = " << water_h(T0,rho).diff(rho) << endl;
    cerr << " = " << water_h(T0,rho).diff(rho).subs(rho==rho0) << endl;


    T0 = 200+273.15;
    cerr << "DENSITY VARIATION at T = " << T0 << endl;

    lst rho_vals;
    rho_vals = 0.0458,0.230,0.460,2.353,4.86,6,10,10.1,15,20,30,50,100,150,200,300,400,500,600,700,800,865,877,868,871,875,878,880,882,883,891,897,924;
    int n = rho_vals.nops();
    int i = 0;
    rho0 = rho_vals[i];
	p = water_p(T0,rho0);
	h = water_h(T0,rho0);
	s = water_s(T0,rho0);
	while(i<n-1){
		ex dpdrho = water_p(T0,rho).diff(rho).subs(rho==rho0);
		ex dhdrho = water_h(T0,rho).diff(rho).subs(rho==rho0);
		ex dsdrho = water_s(T0,rho).diff(rho).subs(rho==rho0);
		rho1 = rho_vals[i+1];
		ex p1 = p + (rho1-rho0) * dpdrho;
		ex h1 = h + (rho1-rho0) * dhdrho;
		ex s1 = s + (rho1-rho0) * dsdrho;

		cerr << "rho= " << setprecision(5) <<setw(8) << ex_to<numeric>(rho0).to_double() << "  ";
		cerr << "p= " << setprecision(4) << setw(8) << ex_to<numeric>(p).to_double() << "  ";
		cerr << "h= " << setprecision(4) << setw(6) << ex_to<numeric>(h).to_double() << "  ";
		cerr << "s= " << setprecision(4) << setw(6) << ex_to<numeric>(s).to_double() << "  ";
		cerr << "dh/drho= " << setw(12) << ex_to<numeric>(dhdrho).to_double() << "  ";
		cerr << "h_ex= " << setw(10) << ex_to<numeric>(h1).to_double() << "  ";

		p = water_p(T0,rho1);
		h = water_h(T0,rho1);
		s = water_s(T0,rho1);

		cerr << "h_co= " << setw(6) << ex_to<numeric>(h).to_double() << "  ";
		cerr << "s_ex= " << setw(10) << ex_to<numeric>(s1).to_double() << "  ";
		cerr << "s_co= " << setw(6) << ex_to<numeric>(s).to_double() << "  ";

		numeric p_err = ex_to<numeric>( (p1 - p)/p * 100 );
		numeric h_err = ex_to<numeric>( (h1 - h)/h * 100 );
		numeric s_err = ex_to<numeric>( (s1 - s)/s * 100 );

		cerr << "err_p = " << setw(8) << setprecision(2) << p_err.to_double() << "%  ";
		cerr << "err_h = " << setw(8) << setprecision(2) << h_err.to_double() << "%  ";
		cerr << "err_s = " << setw(8) << setprecision(2) << s_err.to_double() << "% ";
        cerr << endl;
		rho0=rho1;
        i++;
	}

	/*
    cerr << "p = " << water_p(T1,rho1) << endl;
    cerr << "dp/dT = " << water_p(T,rho1).diff(T).subs(T==T1) << endl;
    cerr << "dp/drho = " << water_p(T1,rho).diff(rho).subs(rho==rho1) << endl;
    cerr << "h = " << water_h(T1,rho1) << endl;
    cerr << "dh/dT = " << water_h(T,rho1).diff(T).subs(T==T1) << endl;
    cerr << "dh/drho = " << water_h(T1,rho).diff(rho).subs(rho==rho1) << endl;
    cerr << "s = " << water_s(T1,rho1) << endl;
    cerr << "ds/dT = " << water_s(T,rho1).diff(T).subs(T==T1) << endl;
    cerr << "ds/drho = " << water_s(T1,rho).diff(rho).subs(rho==rho1) << endl;
	*/
}
