/*

nla-ginac - Tarjan-blocking NLA solver with autodiff
Copyright (C) 2005  John Pye

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#include "symbolicsystem.h"
#include "nnessolver.h"
#include "units.h"
#include "iapws-sat.h"

using namespace GiNaC;
using namespace std;

class PipeSystem: public SymbolicSystem{

	virtual void defineSystem(){

		CONST(rho1,871);
		CONST(T1,200+273.15);

		CONST(m1,0.4);

		//CONST(hpipe,0);
		//CONST(Tamb,300);

		CONST(f,0.001);
		CONST(L,10);
		CONST(D,0.05);

		CONST(K,f * L / D);
		CONST(A1,Pi * D*D/4);
		//CONST(Apipe, Pi*D*L);

        VARIABLE(DP, double(-10.0 * bar/ MPa));
        //CONST(DP,-10.0 * bar / MPa);

		VARIABLE(p1,water_p(T1,rho1));
		VARIABLE(p2,1.0 * bar/MPa);

		VARIABLE(h1,2400);

		VARIABLE(m2,0.01);
		VARIABLE(T2, 300 + 273.15);
		VARIABLE(rho2, 0.379);
		VARIABLE(h2, water_h(T1,0.02));

		//VARIABLE(Qpipe, 0.5 /* kW */);
        LOWERB(T2,0);
		LOWERB(rho2,0);
        LOWERB(p2,0);

		lst ff; ff =
			// Inlet properties
			p1 - water_p(T1,rho1)
			, h1 - water_h(T1,rho1)

			// Pressure drop (momentum)
			//, (p2 - p1)  +  0.5 * K * pow((m1 + m2)/2,2) / pow(A1,2) / rho1
            , DP + 0.5 * K * pow((m1+m2)/2,2) / pow(A1,2) / rho1
            , (p2 - p1) - DP
            //, deltapr - (p2 - p1)

			// Mass conservation
            , m2 - m1
			// Energy absorbed by the flow, from outside ambience
			//, (m1 + m2) / 2 * ( h2 - h1) + hpipe * Apipe * ((T1 + T2)/2  - Tamb)
            , h2 - h1

			// Outlet properties
			, p2 - water_p(T2,rho2)
			, h2 - water_h(T2,rho2)
		;
        setExpressions(ff);

	}

	virtual void applySolverOptions(Solver *solver) const{
		cerr << "PipeSystem::applySolverOptions:..." << endl;
        applyBounds(solver);
        //OPTION(ITSCLX,10);
        //OPTION(STOPCR,2);
        OPTION(JACTYP,0);
        //OPTION(OUTPUT,2);
	}

};

int main(){
    PipeSystem P;
    P.accept(NNESSolver::Instance());
}
