/*

nla-ginac - Tarjan-blocking NLA solver with autodiff
Copyright (C) 2005  John Pye

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#include <cstdlib>
#include <iomanip>
#include <iostream>
#include <stdexcept>
using namespace std;

#include "printblocks.h"
#include "mc13d.h"

/* Table of constant values */

#define FORTRAN_ARRAY(ARRAY,R,C,NR) ARRAY[(C)*NR +(R)]

namespace MC13D
{

    char esc = unsigned(27);

    void printrelmap(const bool *rel, const unsigned &nr, const unsigned &nc){
		unsigned *iperm = new unsigned[nr];
		for(unsigned i=0; i<nr; ++i){
			iperm[i]=i+1;
		}
		printrelmap_perm(rel, nr, nc, iperm);
		delete[] iperm;
	}

	void printrelmap_perm(const bool *rel, const unsigned &nr, const unsigned &nc, const unsigned *iperm){
		cerr << "+";
		for(unsigned c=0; c< nc; ++c){
			cerr << "-";
		}
		cerr << "+" << endl;
		for(unsigned r=0; r< nr; ++r){
			cerr << "|";
			for(unsigned c=0; c< nc; ++c){
				if(FORTRAN_ARRAY(rel,iperm[r]-1,c,nr)){
					cerr << esc << "[31;1m#" << esc<< "[0m";
				}else{
					cerr << esc << "[32;1m+" << esc <<"[0m";
				}
			}
			cerr << "|" << endl;
		}
		cerr << "+";
		for(unsigned c=0; c< nc; ++c){
			cerr << "-";
		}
		cerr << "+" << endl;
	}

    void printblocks(const int &n,const int &num,int *ib,const unsigned *ior,const logical *a){
		unsigned *iperm = new unsigned[n];
		for(int i=0; i<n; ++i){
			iperm[i]=i+1;
		}
		printblocks_perm(n,num,ib,iperm,ior,a);
		delete[] iperm;
	}


    void printblocks_perm(const int &n,const int &num,int *ib,const unsigned *iperm, const unsigned *ior,const logical *a){
        char blank = '|';
        char ex = '#';
        char zer = '+';
        char hold[100];
        // THERE ARE num BLOCKS
        cerr << "There are " << num << " blocks:" << endl;
        
        ib[num] = n + 1;
        int index = 100;
        int iblock = 1;

        for (int i = 1; i <= n; ++i) {
			//bool rowblockstart=false;
            for (int ij = 1; ij <= index; ++ij) {
                hold[ij - 1] = blank;
            }
        
            if (i == ib[iblock - 1]) {
				// this row is the start of a block
                //cerr << endl;
                ++iblock;
            }

            int jblock = 1;
            index = 0;
            for (int j = 1; j <= n; ++j) {
                if (j == ib[jblock - 1]) {
                    ++index;
                    hold[index - 1] = '|';
                    ++jblock;
                }
                ++index;
                int ii = ior[i - 1]-1;
                int jj = ior[j - 1]-1;
                if (FORTRAN_ARRAY(a,iperm[ii]-1,jj,n)) {
                    hold[index - 1] = ex;
                }else{
                    hold[index - 1] = zer;
                }
            }
            
            cerr << setw(4) << (i-1) << " ";

			
			bool ul=false;
			if(i+1==ib[iblock-1]){
				ul=true;
				cerr <<esc << "[4m"; // underline this row
			}

            // display this row
			int col=0;
            for(int i=0;i<index;++i){
                if(hold[i]==ex){
                    cerr << esc << "[1;31m" << hold[i];
                }else if(hold[i]==zer){
                    cerr << esc << "[1;32m" << hold[i];
                }else{
					++col;
					if(ul){
						cerr << esc << "[38;2m|";
					}else{
						cerr << esc << "[39;2m|";
					}
				}
            }
			if(ul){
				cerr << esc << "[38;2m|";
			}else{
				cerr << esc << "[39;2m|";
			}
            cerr << esc << "[0m" << endl;
        }

    }


}// namespace MC13D

