/*

nla-ginac - Tarjan-blocking NLA solver with autodiff
Copyright (C) 2005  John Pye

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#include "system.h"
#include "nnessolver.h"
#include "colebrook-sym.h"

class RoughnessSystem: public System{

	virtual void defineSystem(){

        CONST(eps,0.046e-3);
		VARIABLE(D,0.05);
        CONST(Re,3e5);
		VARIABLE(f,0.02);
		
		LOWERB(D,0);

		lst ff; ff = 
			eps/D - 0.008
			, f - f_ReEd(Re,eps/D)
		;

        setExpressions(ff);

	}

	virtual void applySolverOptions(Solver *solver) const{
		applyBounds(solver);
	}

};

main(){
    RoughnessSystem R;
    R.accept(NNESSolver::Instance());
}
