/**
	Calculate saturation curves AND THEIR DERIVATIVES
*/
/*
freesteam - IAPWS-IF97 steam tables library
Copyright (C) 2004-2005  John Pye

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#include "saturation.h"

#define POWDIFF(VAL,EXP) (EXP)*pow(VAL,EXP-1)

namespace Sat{

	//----------------------------------------------------------------
	// COEFFICIENT TABLE DATA

	/**
		Coefficients for getSatDensWater_T
	*/
	const double B[7]
		= { 0, 1.99274064, 1.09965342, -0.510839303, -1.75493479, -45.5170352, -6.74694450E+05 };

	/**
		Coefficients for getSatDensSteam_T
	*/
	const double C[7]
		= { 0, -2.03150240, -2.68302940, -5.38626492, -17.2991605, -44.7586581, -63.9201063 };


	double deltaf(const double &tau){
		return 1 \
			+ B[1] * pow(tau,1./3)
			+ B[2] * pow(tau,2./3)
			+ B[3] * pow(tau,5./3)
			+ B[4] * pow(tau,16./3)
			+ B[5] * pow(tau,43./3)
			+ B[6] * pow(tau,110./3);
	}

	double deltaftau(const double &tau){
		return 0 \
			+ B[1] * POWDIFF(tau,1./3)
			+ B[2] * POWDIFF(tau,2./3)
			+ B[3] * POWDIFF(tau,5./3)
			+ B[4] * POWDIFF(tau,16./3)
			+ B[5] * POWDIFF(tau,43./3)
			+ B[6] * POWDIFF(tau,110./3);
	}

	double deltag(const double &tau){
		double lndeltag = \
			C[1] * pow(tau,2./6)
			+ C[2] * pow(tau,4./6)
			+ C[3] * pow(tau,8./6)
			+ C[4] * pow(tau,18./6)
			+ C[5] * pow(tau,37./6)
			+ C[6] * pow(tau,71./6);
		return exp(lndeltag);
	}

	double deltagtau(const double &tau){
		double lndeltagtau = \
			C[1] * POWDIFF(tau,2./6)
			+ C[2] * POWDIFF(tau,4./6)
			+ C[3] * POWDIFF(tau,8./6)
			+ C[4] * POWDIFF(tau,18./6)
			+ C[5] * POWDIFF(tau,37./6)
			+ C[6] * POWDIFF(tau,71./6);

		return deltag(tau)*lndeltagtau;
	}
}
