/*

nla-ginac - Tarjan-blocking NLA solver with autodiff
Copyright (C) 2005  John Pye

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#include <sstream>

#include "solver.h"
#include "cstring"

class SymbolicSystem;

using namespace std;

Solver::Solver(){
	iopt["QUIET"]=0;
	iopt["CATCH"]=1;
    iopt["SHOW_SPARSITY"]=0;
	// Nothing to do here
}

void
Solver::visit(System *system){
	stringstream ss;
	ss << "Solver::visit must be overloaded for " << getType() << endl;
	throw runtime_error(ss.str());
}

void
Solver::visitSymbolic(SymbolicSystem *system){
	stringstream ss;
	ss << "Solver::visitSymbolic must be overloaded for " << getType() << endl;
	throw runtime_error(ss.str());
}

void
Solver::analyse(System *system){
	cerr << "Solver::analyse:..." << endl;
	stringstream ss;
	ss << "Solver::analyse must be overloaded for " << getType() << endl;
	throw runtime_error(ss.str());
}

void
Solver::analyseSymbolic(SymbolicSystem *system){
	//cerr << "Solver::analyseSymbolic: downcasting to Solver::analyse..." << endl;
	analyse(system);
}

void
Solver::solve(System *system){
	stringstream ss;
	ss << "Solver::solve must be overloaded for " << getType() << endl;
	throw runtime_error(ss.str());
}

void
Solver::solveSymbolic(SymbolicSystem *system){
	//cerr << "Solver::solveSymbolic: downcasting to Solver::solve..." << endl;
	solve(system);
	//throw runtime_error("Solver::solveSymbolic: must be overloaded");
}

bool
ltstr::operator()(const char *a, const char *b) const{
    if(strcmp(a,b)<0)return true;
    return false;
}

void
Solver::setOption(const char *name,const int &val){
    if(iopt.find(name)!=iopt.end()){
        if(!iopt["QUIET"]){
			cerr << "Setting value for option " << name << " = " << val << endl;
		}
        iopt[name]=val;
    }else{
        try{
            setOption(name,double(val));
        }catch(exception &e){
            stringstream ss;
            ss << "Solver::setOption(" << name << ",int " << val << "): " << e.what() << endl;
            ss << "Valid integer options:" << endl;
            ss << "----------------------" ;
            IntegerOptionMap::iterator i;
            int l=0;
            for(i=iopt.begin(); i!=iopt.end(); ++i){
                if(l++%5==0)ss << endl; else ss << ", ";
                ss << (*i).first << " = " << (*i).second;
            }
            ss << endl;
            throw runtime_error(ss.str());
        }
    }
}

void
Solver::setOption(const char *name, const double &val){
    if(dopt.find(name)!=dopt.end()){

		if(!iopt["QUIET"]){
			cerr << "Setting value for double-precision option " << name << " = " << val << endl;
		}
        dopt[name]=val;
    }else{
        stringstream ss;
        ss << "Solver::setOption(" << name << ",double " << val << "): invalid option '" << name << "'" << endl << endl;
        ss << "Valid double options:" << endl;
        ss << "----------------------";
        DoubleOptionMap::iterator i;
        int l=0;
        for(i=dopt.begin(); i!=dopt.end(); ++i){
            if(l++%5==0)ss << endl; else ss << ", ";
            ss << (*i).first << " = " << (*i).second;
        }
        ss << endl;
        throw runtime_error(ss.str());
    }
}

int
Solver::getIntegerOption(const char *name) const{
	IntegerOptionMap::const_iterator i;
	i = iopt.find(name);
	if(i!=iopt.end()){
		return (*i).second;
	}
	stringstream ss;
	ss << "Solver::getOption: invalid option '" << name << "'";
	throw runtime_error(ss.str());
}

System *
Solver::getSystem() const{
	return system;
}

const char *
Solver::getType() const{
    return "Solver";
}
