/*

nla-ginac - Tarjan-blocking NLA solver with autodiff
Copyright (C) 2005  John Pye

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#ifndef SOLVER_H
#define SOLVER_H

#include <map>

#include "system.h"
#include "symbolicsystem.h"

class ltstr
{
public:
    bool operator()(const char *a,const char *b) const;
};

typedef std::map<const char *,int,ltstr> IntegerOptionMap;
typedef std::map<const char *,double,ltstr> DoubleOptionMap;

class Solver
{
public:

	virtual void visit(System *system);
	virtual void visitSymbolic(SymbolicSystem *system);

	virtual void analyse(System *system);
	virtual void analyseSymbolic(SymbolicSystem *system);
	virtual void solve(System *system);
	virtual void solveSymbolic(SymbolicSystem *system);
	
	virtual void applyLowerBound(const unsigned&,const double&) =0;
	virtual void applyUpperBound(const unsigned&,const double&) =0;

	// clever option setting, checks for existence of option first:
    void setOption(const char *,const int &);
    void setOption(const char *,const double &);

	int getIntegerOption(const char *) const;

	virtual ~Solver(){}

	virtual System *getSystem() const;
    virtual const char *getType() const;

protected:
	Solver();

protected:

	System *system;

	IntegerOptionMap iopt;
    DoubleOptionMap dopt;

	static const char esc = unsigned(27);
};

#endif
