#include <list>
#include <vector>
#include <iostream>
#include <algorithm>

using namespace std;

#include "tree.h"
#include "steward.h"
#include "cycle.h"
#include "tearsets.h"

// Access an 2-D array of double in the Fortran way...
#define FORTRAN_ARRAY(ARRAY,R,C,NR) ARRAY[(C)*NR +(R)]

void steward_descend(tree<int> &tr, tree<int>::iterator curr, CycleSet &loops, const int &n, const bool *adjac);

/// Return an incidence matrix mapping the set of equations onto the set of loops given an adjacency matrix, following the algorithm of Steward, 1965.
CycleSet
steward(const int n, const bool *adjac){

	tree<int> tr;
	CycleSet loops;

	tree<int>::iterator top, root, curr, par;
	top = tr.begin();

	root = tr.insert(top, 0);
	curr = root;

	steward_descend(tr, root, loops, n, adjac);

	//unsigned l = loops.size();

	/*
	cerr << char(27) << "[1m" << "LOOPS" << char(27) << "[0m (" << l << ")" << endl;
	CycleSet::iterator i;
	for(i=loops.begin(); i!=loops.end(); ++i){
		cerr << *i << endl;
	}
	*/
	//cerr << "end of loops" << endl;

	// unsigned m = l * n;
	//cerr << "size of incid matrix = " << m << endl;

	/*
	bool *incid = new bool[m];
	for(unsigned i=0;i<m; ++i){
		incid[i]=false;
	}

	//cerr << "Incid matrix set to zero" << endl;

	CycleSet::iterator r;
	int ri;
	for(r = loops.begin(), ri=0; r!=loops.end(); ++r, ++ri){
		//cerr << "ROW = " << ri << endl;
		//cerr << "LOOP = " << *r << endl;

		cycle::iterator c;
		cycle loop = *r;

		for(c = loop.begin(); c != loop.end(); ++c){
			//cerr << "COL = " << *c << endl;
			FORTRAN_ARRAY(incid,ri,*c,l) = true;
		}
	}
	*/
	//cerr << "done creating incid matrix" << endl;

	/*
	cerr << char(27) << "[1m" << "EQUATION TO LOOP INCIDENCE MATRIX" << char(27) << "[0m" << endl;
	for(unsigned r=0;r<l; r++){
		for(int c=0; c<n; c++){
			if(FORTRAN_ARRAY(incid,r,c,l)){
				cerr << "X";
			}else{
				cerr << ".";
			}
		}
		cerr << endl;
	}
	*/

	return loops;
}

ostream& operator<< (ostream& os , const tree<int> & tr) {
	tree<int>::iterator i=tr.begin();
	while(i!=tr.end()) {
		os << "    ";
		for(int j=0; j<tr.depth(i); ++j)
			os << ".";
		os << (*i) << endl;
		++i;
	}
	return os;
}

/**
	Recursive traversal function
*/
void steward_descend(tree<int> &tr, tree<int>::iterator curr, CycleSet &loops, const int &n, const bool *adjac){

	// the row we're working on:
	int r = *curr;
	tree<int>::iterator par, top;
	top=tr.begin();

	// cerr << "New row, r = " << r << endl;

	//cerr << "Tree is currently: " << endl;
	//cerr << tr;

	// for the current row, add child node, and label loops as required
	for(int c=0; c<n; ++c){
		if(c==r)continue; // ignore the diagonal
		if(FORTRAN_ARRAY(adjac,r,c,n)){
			// cerr << "Column " << c << endl;
			par=curr;
			bool is_loop=false;
			cycle loop;
			//cerr << "par = " << *par << ", top = " << *(tr.begin()) << endl;
			while(par!=tr.begin()){
				// cerr << "with par = " << *par << ", loop = " << loop << endl;
				loop.push_back(*par); // first item in loop is current node
				if(*par==c){ // the parent is already in the tree
					// cerr << "loop found, breaking" << endl;
					is_loop=true; // therefore we have a loop
					break;
				}
				// cerr << "moving up the tree" << endl;
				// continue up the tree
				par=tr.parent(par);
			} // while
			if(is_loop){
				if(loops.find(loop) !=loops.end()){
					//cerr << "Loop already found" << endl;
				}else{
					loops.insert(loop);
				}
			}
			tree<int>::iterator child = tr.append_child(curr,c);
			// cerr << "Added node " << *child << " under " << *curr << endl;
			if(!is_loop){
				//cerr << "Descending since it's not a node in a loop (yet)" << endl;
				steward_descend(tr, child, loops, n,adjac);
			}
			// else continue with the other nodes at this level
		}
	}
	// cerr << "Done at this level" << endl;
	// finished this row, all done.
}
