#ifndef STEWARD_H
#define STEWARD_H

#include "cycle.h"

/**
	Return an incidence matrix mapping the set of equations onto the set of loops
	given an adjacency matrix, following the algorithm of Steward, 1965.

	Steward, Donald V, "Partioning and Tearming Systems of Equations",
	J SIAM Numer Anal. Ser B, Vol 2, No 2, 1965.

	@param n size of the current block (use MC13D to decompose a matrix into blocks)
	@param adjac matrix of booleans mapping variables onto equations.
	@return matrix mapping equations onto loops

	A cost calculation can be applied to the output matrix to determine
	the optimal tear.
*/
CycleSet steward(const int n, const bool *adjac);

#endif
