/*

nla-ginac - Tarjan-blocking NLA solver with autodiff
Copyright (C) 2005  John Pye

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#ifndef TARJAN_H
#define TARJAN_H

#define TARJAN_TEARING 1

#include <ginac/ginac.h>

#include "tarjansystem.h"
#include "solver.h"
#include "eqprocess.h"

class SymbolicSystem;

/**
	This class will create a mapping to nonzero-diagonal form (if required)
	then perform blocking on the diagonalised relationship matrix using the Tarjan
	algorith,

	For each block identified, a TarjanSystem will be created such that
	it grabs its equations from this class when it comes time to solve.

	This class, when solving the blocks, will keep track of the solved variables
	and substitute them through the later blocks. Then, when the TarjanSystems
	request their equations, they will be given equations simplified to contain
	only the required variables.
*/
class TarjanSolver : public Solver
{

public:
	virtual void visitSymbolic(SymbolicSystem *s);

	virtual void analyseSymbolic(SymbolicSystem *s);
	virtual void solveSymbolic(SymbolicSystem *s);

	virtual void applyLowerBound(const unsigned&,const double&);
	virtual void applyUpperBound(const unsigned&,const double&);
    virtual const char *getType() const;

	EqProcess &getEqProcess(const unsigned int &i);
	const unsigned getNumEqs() const;
	// const GiNaC::ex &getCurrentEquation(const unsigned &i) const;
	virtual void eliminateVariable(const SymbolicVariable *var, const double &val);

public:

	TarjanSolver();
	virtual ~TarjanSolver();

	const GiNaC::ex getExpression(const unsigned&) const;
	const GiNaC::lst getExpressions() const;
	
	SymbolicSystem *getBaseSystem();

private:
	std::vector<TarjanSystem> subsystems; ///< index in the array of equations of the start of each block
	std::vector<Solver *> subsolvers;

	std::vector<EqProcess> equations;

	Solver *subsolver;
	SymbolicSystem *system;
	
	void printbase() const;

	double *boundl;
	double *boundu;
};

#endif // TARJAN_H
