/*

nla-ginac - Tarjan-blocking NLA solver with autodiff
Copyright (C) 2005  John Pye

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#ifndef TARJANSYSTEM_H
#define TARJANSYSTEM_H

#include "system.h"
#include "symbolicsystem.h"
#include "eqprocess.h"

#include <vector>

class TarjanSolver;

/**
	Class to hold the equations corresponding to one block in a system of equations
	as found by the TarjanSolver. 
	
	This allows the block itself to then be solved by any chosen Solver object, or
	to be solved recursively by a TearingSolver, etc.
*/
class TarjanSystem : public SymbolicSystem{

public:
	TarjanSystem(TarjanSolver *tarjansolver, const unsigned &begin, const unsigned &end);
    virtual ~TarjanSystem();

	virtual void acceptAnalyse(Solver *solver);
	virtual void acceptSolve(Solver *solver);

	virtual const char *getType() const;
    
	void eliminateBaseVariable(const int &e,const int &v, const double &val);

	virtual void defineSystem();
	virtual void doDefineSystem(Solver &solver);

	virtual void applySolverOptions(Solver *solver) const;

	void printbase() const;

	void getSolverEquations();

private:
	TarjanSolver *tarjansolver;
	std::vector<EqProcess *> baseequations;

	void notifySolution();

};

#endif
