#include <iostream>
#include <sstream>

#include "tearingsystem.h"
#include "tearingsolver.h"
#include "solver.h"

using namespace std;

TearingSystem::~TearingSystem(){
	cerr << "DESTROYING TEARINGSYSTEM" << endl;
}

TearingSystem::TearingSystem(TearingSolver &ts, const SymbolicSystem &system, const TornEqsMap &torneqs)
		: tearingsolver(ts)
{
	cerr << "NEW TEARINGSYSTEM" << endl;

	TornEqsMap::const_iterator ti;
	for(ti=torneqs.begin(); ti!=torneqs.end(); ++ti){
		const unsigned &eq = (*ti).first;
		Variable *var = system.getVar((*ti).first);

		cerr << "Tear variable: " << var->getName() << endl;
		cerr << "Torn equation: " << system.getExpression(eq) << endl;

		inheritVariable(var);
	}
	this->eqs=torneqs.size();
	this->vars=torneqs.size();
}

const char *
TearingSystem::getType() const{
	return "TearingSystem";
}

/**
	The NNES Solver will come and visit us at some point...
*/
void
TearingSystem::accept(Solver *solver){
	doDefineSystem(*solver);
	solver->visit(this);
	cerr << "TearingSystem::accept: done" << endl;
}

/**
	Get residuals by substituting the given values of the tear variable
	then solving the system and returning the error across the tear.
*/
void 
TearingSystem::ResFn(int *overfl, const unsigned *n1, double* res, const double* y){
	cerr << "TearingSystem::ResFn..." << endl;
	*overfl=0;

	try{
		// the TearingSolver knows how system needs to be solved once the torn variables have been sorted out
		tearingsolver.getTearResiduals(y, res);

	}catch(exception &e){
		stringstream ss;
		ss << "TearingSystem::ResFn: " << e.what() << endl;
		throw runtime_error(ss.str());
		// exit(1);
		*overfl=1;
	}
}

/**
	Set up the system for use by NNES Solver
*/
void
TearingSystem::doDefineSystem(const Solver &solver){
	cerr << "NEED TO SET UP A 'REL' MATRIX..." << endl;
}

void
TearingSystem::defineSystem(){
	// nothing here
}

void 
TearingSystem::applySolverOptions(Solver *solver) const{
	// what to do
	cerr << "TearingSystem::applySolverOptions:..."<< endl;
	OPTION(JACTYP,2); // numerical Jacobian, anything else would be too hard :)
}
