#ifndef TEARINGSYSTEM_H
#define TEARINGSYSTEM_H

#include "symbolicsystem.h"

class TearingSolver;

/**
	This class exists so that torn variables and tear residuals from a larger system
	can be exposed to an NLA solver such as NNES, and then solved for iteratively.

	It's not a SymbolicSystem because the Jacobian can't be prepared symbolic for the tearing loops
	since there are variables in the loop which must be solved numerically.
*/
class TearingSystem : public System{

public:
	TearingSystem(TearingSolver &ts, const SymbolicSystem &s, const TornEqsMap &torneqs);
	virtual ~TearingSystem();

	virtual const char *getType() const;

	virtual void defineSystem();
	virtual void doDefineSystem(const Solver &solver);

	virtual void accept(Solver *solver);
	virtual void applySolverOptions(Solver *solver) const;

	virtual void ResFn(int *overfl, const unsigned *n1, double* res, const double* y);
	
private:
	TearingSolver &tearingsolver;
};

#endif // TEARINGSYSTEM_H
