#include "tearsets.h"
#include "tree.h"

using namespace std;

#include "cycle.h"

void tearset_descend(CycleSet::iterator i, const CycleSet &loops, set<unsigned> addable_eqs, set<unsigned> tearing_eqs, const unsigned &eq, vector<TearSet> &ts);

/// Home-baked algorithm for finding complete, non-redundant tear sets
vector<TearSet>
tearsets(const CycleSet &loops, const unsigned &n){

	vector<TearSet> ts;

	cycle loop;
	cycle::iterator j;
	
	// try all of the equations in the first loop:
	loop = *(loops.begin());
	set<unsigned> addable_eqs;
	set<unsigned> tearing_eqs;

	// initiall all equations are addable:
	for(unsigned k=0; k<n; k++){
		addable_eqs.insert(k);
	}
	
	for(j=loop.begin(); j!=loop.end(); ++j){
		// cerr << "Starting with equation " << *j << endl;
		tearset_descend(loops.begin(), loops, addable_eqs, tearing_eqs, *j, ts);
	}

	return ts;
}

void
tearset_descend(CycleSet::iterator i
		, const CycleSet &loops
		, set<unsigned> addable_eqs
		, set<unsigned> tearing_eqs
		, const unsigned &eq
		, vector<TearSet> &ts
){
	// for the given loop, remove all other equations other than eq the set of addable equations
	cycle loop = *i;
	cycle::iterator j;

	// for equation eq, remove all other equations in this loop from the list of addable equations
	for(j=loop.begin(); j!=loop.end(); ++j){
		if(*j == eq) continue;
		set<unsigned>::iterator ae;
		if((ae = addable_eqs.find(*j)) != addable_eqs.end()){
			addable_eqs.erase(ae);
		}
	}

	//cerr << "Adding eq " << eq << " to the tearset" << endl;
	tearing_eqs.insert(eq);

	// move to the next loop
	while(1){
		++i; 
		if(i == loops.end()){
			// if we are here, it is the last row and we have torn each loop once
			set<unsigned>::iterator tei;
			/*
			cerr << "Completed tearset: ";
			for(tei=tearing_eqs.begin(); tei!=tearing_eqs.end(); ++tei){
				cerr << *tei << " ";
			}
			cerr << endl;
			*/
			ts.push_back(tearing_eqs);
			return;
		}
		loop = *i;
		
		// is this loop torn already?
		bool already_torn=false;
		for(j=loop.begin(); j!=loop.end(); ++j){
			if(tearing_eqs.find(*j) != tearing_eqs.end()){
				// yes
				already_torn=true;
			}
		}

		if(!already_torn){
			bool found_addable_equation = false;
			for(j=loop.begin(); j!=loop.end(); ++j){
				if(addable_eqs.find(*j) != addable_eqs.end()){
					// the equation is addable
					tearset_descend(i,loops,addable_eqs,tearing_eqs,*j, ts);
				}
			}
			if(!found_addable_equation){
				//cerr << "This tearset failed." << endl;
			}
			return; // don't descend again
		}
		// descend without adding an equation...
	}
	// if we are here then we are just quietly returning back up to top level
}
				
			





