/*

nla-ginac - Tarjan-blocking NLA solver with autodiff
Copyright (C) 2005  John Pye

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
/**
	Symbolic colebrook equation and derivative
*/

#include <ginac/ginac.h>
#include <iostream>

#include "viscosity-sym.h"
#include "viscosity.h"

using namespace GiNaC;
using namespace std;


/// SATURATION STEAM TABLES: ENTHALPY

static ex mu_Trho_eval(const ex &T, const ex &rho){
	ex T1 = T.evalf();
	ex rho1 = rho.evalf();
	if(is_a<numeric>(T1) && is_a<numeric>(rho1)){
		double T2 = ex_to<numeric>(T1).to_double();
		double rho2 = ex_to<numeric>(rho1).to_double();
		//cerr << "mu_Trho_eval: T = " << T << ", rho = " << rho;
		return Viscosity::dynvisc(T2,rho2);
		//cerr << " --> mu = " << mu << endl;
		//return mu;
	}else{
		return mu_Trho(T,rho).hold();
	}
}

static ex mu_Trho_diff(const ex &T, const ex &rho, unsigned diff_param){
	switch(diff_param){
		case 0:
			return mu_Trho_dT(T,rho);
		case 1:
			return mu_Trho_drho(T,rho);
		default:
			throw runtime_error("mu_Trho_diff: invalid diff_param");
	}
}

REGISTER_FUNCTION(mu_Trho, eval_func(mu_Trho_eval).derivative_func(mu_Trho_diff));

static ex mu_Trho_dT_eval(const ex &T, const ex &rho){
	ex T1 = T.evalf();
	ex rho1 = rho.evalf();
	if(is_a<numeric>(T1) && is_a<numeric>(rho1)){
		double T2 = ex_to<numeric>(T1).to_double();
		double rho2 = ex_to<numeric>(rho1).to_double();
		return Viscosity::dynvisc_dT(T2,rho2);
	}else{
		return mu_Trho_dT(T,rho).hold();
	}
}

static ex mu_Trho_drho_eval(const ex &T, const ex &rho){
	ex T1 = T.evalf();
	ex rho1 = rho.evalf();
	if(is_a<numeric>(T1) && is_a<numeric>(rho1)){
		double T2 = ex_to<numeric>(T1).to_double();
		double rho2 = ex_to<numeric>(rho1).to_double();
		return Viscosity::dynvisc_drho(T2,rho2);
	}else{
		return mu_Trho_drho(T,rho).hold();
	}
}

REGISTER_FUNCTION(mu_Trho_dT, eval_func(mu_Trho_dT_eval));
REGISTER_FUNCTION(mu_Trho_drho, eval_func(mu_Trho_drho_eval));

