/*

nla-ginac - Tarjan-blocking NLA solver with autodiff
Copyright (C) 2005  John Pye

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#ifndef VISCOSITY_H
#define VISCOSITY_H

#ifndef NUM_DEFINED
// When integrating with the IF97 stuff, Num is already defined so don't redefine it...
typedef double Num;
#endif

namespace IAPWS
{
	const Num Tstar_visc = 647.226;
	const Num rhostar_visc = 317.763;
	const Num mustar_visc = 55.071e-6;

	enum Limits{
		VISC_N0_COUNT=4
		,VISC_COUNT=19
	};
	
	const Num VISC_N0[VISC_N0_COUNT] = { 1, 0.978197, 0.579829, -0.202354 };
		
	const Num VISC_I[VISC_COUNT] = { 0, 1, 4, 5, 0, 1, 2, 3, 0, 1, 2, 0, 1, 2, 3, 0, 3, 1, 3 };
	const Num VISC_J[VISC_COUNT]= { 0, 0, 0, 0, 1, 1, 1, 1, 2, 2, 2, 3, 3, 3, 3, 4, 4, 5, 6 };
	
	const Num VISC_N[VISC_COUNT] = { 0.5132047, 0.3205656, -0.7782567, 0.1885447, 0.2151778, 0.7317883, 1.241044,
		1.476783, -0.2818107, -1.070786, -1.263184, 0.1778064, 0.460504, 0.2340379, -0.4924179, -0.0417661,
		0.1600435, -0.01578386, -0.003629481 
	};

	Num mu0(const Num &tau);
	Num mu0tau(const Num &tau);

	Num mu1(const Num &tau,const Num&delta);
	Num mu1tau(const Num &tau,const Num&delta);
	Num mu1delta(const Num &tau,const Num&delta);

	Num mu2(const Num &tau,const Num&delta);
	Num mu2tau(const Num &tau,const Num&delta);
	Num mu2delta(const Num &tau,const Num&delta);

	Num mu(const Num &tau,const Num &delta);
	Num mutau(const Num &tau,const Num &delta);
	Num mudelta(const Num &tau,const Num &delta);
}

class Viscosity{
public:
	static Num dynvisc(const Num &T,const Num &rho);
	static Num dynvisc_dT(const Num &T,const Num &rho);
	static Num dynvisc_drho(const Num &T,const Num &rho);
};
	
#endif // VISCOSITY_H
