/*

nla-ginac - Tarjan-blocking NLA solver with autodiff
Copyright (C) 2005  John Pye

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#include "system.h"
#include "roulette.h"
#include "nnessolver.h"
#include "viscosity-sym.h"

using namespace GiNaC;

class ViscositySystem: public System{

	virtual void defineSystem(){

		VARIABLE(rho1,850);
		CONST(T1,250+273.15);

        VARIABLE(mu1,1e-5);

		CONST(p1,10);
		
		LOWERB(mu1,0);
		UPPERB(mu1,1);

		lst ff; ff = 
			p1 - water_p(T1,rho1)
			, mu1 - mu_Trho(T1,rho1)
		;
        setExpressions(ff);

	}

	virtual void applySolverOptions(Solver *solver) const{
        //applyBounds(solver);
        //OPTION(ITSCLX,10);
        //OPTION(ITSCLF,9);
        //OPTION(STOPCR,2);
        OPTION(JACTYP,0);
        //OPTION(FDTOLJ,2e-4);
        OPTION(OUTPUT,0);
	}

};

main(){
    ViscositySystem S;
	Roulette R;
    S.accept(&R);
}
