/*

nla-ginac - Tarjan-blocking NLA solver with autodiff
Copyright (C) 2005  John Pye

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#include "symbolicsystem.h"
#include "nnessolver.h"
#include "units.h"
#include "iapws-sat.h"

using namespace std;
using namespace GiNaC;

class WaterSystem : public SymbolicSystem{

	virtual void defineSystem(){
		cerr << "WaterSystem::defineSystem:..." << endl;

		CONST(p,1.0 * bar/ MPa);
		CONST(h,2400);

		VARIABLE(rho,998);
		VARIABLE(T,500);


        lst f;
        f =
			p - water_p(rho,T)
			, h - water_h(rho,T)
		;
        setExpressions(f);

	}

	virtual void applySolverOptions(Solver *solver) const{
		OPTION(JACTYP,0);
	}

};


int main(){
	try{

		WaterSystem W;
		W.accept(NNESSolver::Instance());

    }catch(exception &e){
        cerr << "MAIN: " << e.what() << endl;
        exit(1);
    }catch(...){
        cerr << "MAIN: uncaught unknown exception" << endl;
    }

    return 0;
}
